;;;=================================================================
;;;
;;; RECH.LSP V1.10
;;;
;;; Recherche un texte dans le dessin, dans les xrefs
;;;
;;; Avec l'aide prcieuse de (gile) sur CadXP sur ces deux sujets
;;; http://www.cadxp.com/sujetXForum-18995.htm
;;; http://www.cadxp.com/sujetXForum-19080.htm
;;;
;;; Copyright (C) Patrick_35
;;;
;;;=================================================================

(defun patrick:rechercher(str wcm / bg bl cac cen cod doc ent flag hd lay lst lst_fen old_error sel spa str tot xreo
				    *errpat* acti_fen affiche calque conv gele hexdec mxm mxv ouvrir_dessin_dbx reche rech_att
				    rech_bl rech_tab rech_txt rech_xref refgeom revrefgeom transnested trp unformat vxs vxv wcs2pcs)

  ;-----------------------------------------------------------------
  ; Gestion des erreurs
  ;-----------------------------------------------------------------

  (defun *errpat* (msg)
    (or (member (strcase msg) '("FUNCTION CANCELLED" ""QUIT / EXIT ABORT"" "FONCTION ANNULEE" "QUITTER / SORTIR ABANDON"))
      (princ (strcat "\nErreur : " msg))
    )
    (and spa (vla-put-mspace spa :vlax-true))
    (and old_patrick:recherche
	 (vlax-read-enabled-p old_patrick:recherche)
	 (redraw (vlax-vla-object->ename old_patrick:recherche) 4)
    )
    (and (not xreo) dbx (vlax-release-object dbx))
    (vla-endundomark doc)
    (setq *error* old_error)
    (princ)
  )

  ;-----------------------------------------------------------------
  ; Convertion Hexadecimal ---> Decimal
  ;-----------------------------------------------------------------

    (defun hexdec (nb / r i s) 
      (and (= (type nb) 'INT)
        (setq nb (itoa nb))
      )
      (setq r 0 i 0)
      (while (and (= (type nb) 'STR) (< i (strlen nb)))
        (setq i (1+ i)
	      s (strcase (substr nb i 1 ))
	      r (+ (lsh r 4)
		   (- (ascii s)
		      (if (<= s "9")
			48
			55
		      )
		   )
		)
        )
      )
    )

  ;-------------------------------------------------------------------------
  ; Rcuperer la valeur String d'un Mtext
  ; Extrait du lisp de Custom Stuff StripMtext v3.07
  ; http://www.users.qwest.net/~sdoman/
  ; Modifi par Patrick_35 pour correction d'un bug+formattage des caractres spciaux
  ;-------------------------------------------------------------------------

  (defun UnFormat (Mtext Formats / All Format1 Format2 Format3 Pos Text Str)
    (and
      Mtext
      Formats
      (= (type Mtext) 'STR)
      (= (type Formats) 'STR)
      (setq Formats (strcase Formats))
      (setq Text "")
      (setq All T)
      (if (= Formats "*")
	(setq Formats "S"
	      Format1 "\\[LO`~]"
	      Format2 "\\[ACFHQTW]"
	      Format3 "\\P"
	)
	(progn
	  (setq Format1 "" Format2 "" Format3 "")
	  (foreach item '("L" "O" "~")
	    (if (vl-string-search item Formats)
	      (setq Format1 (strcat Format1 "`" item))
	      (setq All nil)
	    )
	  )
	  (if (= Format1 "")
	    (setq Format1 nil)
	    (setq Format1 (strcat "\\[" Format1 "]"))
	  )
	  (foreach item '("A" "C" "F" "H" "Q" "T" "W")
	    (if (vl-string-search item Formats)
	      (setq Format2 (strcat Format2 item))
	      (setq All nil)
	    )
	  )
	  (if (= Format2 "")
	    (setq Format2 nil)
	    (setq Format2 (strcat "\\[" Format2 "]"))
	  )
	  (if (vl-string-search "P" Formats)
	    (setq Format3 "\\P")
	    (setq Format3 nil All nil)
	  )
	  T
	)
      )
      (while (/= Mtext "")
	(cond
	  ((wcmatch (strcase (setq Str (substr Mtext 1 2))) "\\[\\{}]")
	    (setq Mtext (substr Mtext 3)
		  Text   (strcat Text Str)
	    )
	  )
	  ((and All (wcmatch (substr Mtext 1 1) "[{}]"))
	    (setq Mtext (substr Mtext 2))
	  )
	  ((and Format1 (wcmatch (strcase (substr Mtext 1 2)) Format1))
	    (setq Mtext (substr Mtext 3))
	  )
	  ((and Format2 (wcmatch (strcase (substr Mtext 1 2)) Format2))
	    (setq Mtext (substr Mtext (+ 2 (if (vl-string-search ";" Mtext)(vl-string-search ";" Mtext) 0))))
	  )
	  ((and Format3 (wcmatch (strcase (substr Mtext 1 2)) Format3))
	    (if
	      (or
		(= " " (substr Text (if (zerop (strlen Text)) 1 (strlen Text))))
		(= " " (substr Mtext 3 1))
	      )
	      (setq Mtext (substr Mtext 3))
	      (setq Mtext (substr Mtext 3) Text (strcat Text " "))
	    )
	  )     
	  ((and (vl-string-search "S" Formats)(wcmatch (strcase (substr Mtext 1 2)) "\\S"))
	    (setq Str   (substr Mtext 3 (- (vl-string-search ";" Mtext) 2))
		  Text  (strcat Text (vl-string-translate "#^\\" "/^\\" Str))
		  Mtext (substr Mtext (+ 4 (strlen Str)))
	    )
	  )
	  (1
	    (setq Text (strcat Text (substr Mtext 1 1))
		  Mtext (substr Mtext 2)
	    )
	  )
	)
      )
    )
    (while (setq Pos (vl-string-search "\\U+" Text))
      (setq Str (substr Text (1+ pos) 7)
	    Text (vl-string-subst (chr (hexdec (substr Str 4))) Str Text))
    )
    (vl-string-trim " " Text)
  )

  ;-----------------------------------------------------------------
  ; Ouvrir un dessin sans l'afficher dans Autocad
  ;-----------------------------------------------------------------

  (defun ouvrir_dessin_dbx(dwg / dbx)
    (if (< (atoi (substr (getvar "ACADVER") 1 2)) 16)
      (setq dbx (vlax-create-object "ObjectDBX.AxDbDocument"))
      (setq dbx (vlax-create-object (strcat "ObjectDBX.AxDbDocument." (substr (getvar "ACADVER") 1 2))))
    )
    (vla-open dbx dwg)
    dbx
  )

  ;; WCS2PCS (gile)
  ;; Traduit les coordonnes d'un point du SCG dans le SC de l'espace
  ;; papier correspondant  la fentre spcifie
  ;; (WCS2PCS pt vp) est quivalent  (trans (trans pt 0 2) 2 3) avec vp active
  ;;
  ;; Arguments
  ;; pt : un point
  ;; vp : la fentre (ename ou vla-object)

  (defun WCS2PCS (pt vp / elst ang nor scl mat)
    (and (= (type vp) 'VLA-OBJECT)
      (setq vp (vlax-vla-object->ename vp))
    )
    (setq pt (trans pt 0 0)
	  elst (entget vp)
	  ang (cdr (assoc 51 elst))
	  nor (cdr (assoc 16 elst))
	  scl (/ (cdr (assoc 41 elst)) (cdr (assoc 45 elst)))
	  mat (mxm
		(list (list (cos ang) (- (sin ang)) 0.0)
		      (list (sin ang) (cos ang) 0.0)
		      '(0.0 0.0 1.0)
		)
		(mapcar (function (lambda (v) (trans v nor 0 T)))
			'((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))
		)
	      )
    )
    (mapcar '+
	    (vxs (mxv mat (mapcar '- pt (cdr (assoc 17 elst)))) scl)
	    (vxs (cdr (assoc 12 elst)) (- scl))
	    (cdr (assoc 10 elst))
    )
  )

  ;; VXS (gile)
  ;; Retourne le produit d'un vecteur par un scalaire
  ;;
  ;; Arguments : un vecteur et un rel

  (defun vxs (v s) (mapcar (function (lambda (x) (* x s))) v))

  ;; VXV (gile)
  ;; Retourne le produit scalaire (rel) de deux vecteurs
  ;;
  ;; Arguments : deux vecteurs

  (defun vxv (v1 v2) (apply '+ (mapcar '* v1 v2)))

  ;; TRP
  ;; transpose une matrice -Doug Wilson-
  ;;
  ;; Argument : une matrice

  (defun trp (m) (apply 'mapcar (cons 'list m)))

  ;; MXV
  ;; Applique une matrice de transformation  un vecteur -Vladimir Nesterovsky-
  ;;
  ;; Arguments : une matrice et un vecteur

  (defun mxv (m v)
    (mapcar (function (lambda (r) (vxv r v))) m)
  )

  ;; MXM
  ;; Multiple (combine) deux matrices -Vladimir Nesterovsky-
  ;;
  ;; Arguments : deux matrices

  (defun mxm (m q)
    (mapcar (function (lambda (r) (mxv (trp q) r))) m)
  )

  ;; RefGeom (gile)
  ;; Retourne une liste dont le premier lment est une matrice de transformation
  ;; (rotation, chelles, mormale) de dimension 3X3 et le second le point
  ;; d'insertion de l'objet dans son "parent" (xref, bloc ou espace)

  (defun RefGeom (ename / elst ang norm)
    (setq elst (entget ename)
	  ang (cdr (assoc 50 elst))
	  norm (cdr (assoc 210 elst))
    )
    (list
      (mxm
	(mapcar (function (lambda (v) (trans v 0 norm T)))
		'((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))
	)
        (mxm
	  (list (list (cos ang) (- (sin ang)) 0.0)
		(list (sin ang) (cos ang) 0.0)
		'(0.0 0.0 1.0)
	  )
	  (list (list (cdr (assoc 41 elst)) 0.0 0.0)
		(list 0.0 (cdr (assoc 42 elst)) 0.0)
		(list 0.0 0.0 (cdr (assoc 43 elst)))
	  )
	)
      )
      (trans (cdr (assoc 10 elst)) norm 0)
    )
  )

  ;; RevRefGeom (gile)
  ;; Fonction inverse de RefGeom

  (defun RevRefGeom (ename / entData ang norm mat)
    (setq entData (entget ename)
	  ang (- (cdr (assoc 50 entData)))
	  norm (cdr (assoc 210 entData))
    )
    (list
      (setq mat
	(mxm
	  (list (list (/ 1 (cdr (assoc 41 entData))) 0.0 0.0)
		(list 0.0 (/ 1 (cdr (assoc 42 entData))) 0.0)
		(list 0.0 0.0 (/ 1 (cdr (assoc 43 entData))))
	  )
	  (mxm
	    (list (list (cos ang) (- (sin ang)) 0.0)
		  (list (sin ang) (cos ang) 0.0)
		  '(0.0 0.0 1.0)
	    )
	    (mapcar (function (lambda (v) (trans v norm 0 T)))
		    '((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))
	    )
	  )
	)
      )
      (mapcar '- (mxv mat (trans (cdr (assoc 10 entData)) norm 0)))
    )
  )

  ;; TransNested (gile)
  ;; Convertit les coordonnes d'un point entre le SCG ou le SCU et le SCR -systme de
  ;; coordones d'une rfrence (xref ou bloc) quelque soit son niveau d'imbrication-
  ;;
  ;; Arguments
  ;; pt : le point  convertir
  ;; rlst : la liste des entits "parents" de la plus imbriqu  celle insre dans
  ;; l'espace courant -indentique  (last (nentsel)) ou (last (nentselp))
  ;; from to : comme avec trans : 0 pour le SCG, 1 pour le SCU courant, 2 pour le SCR
  ;; (avec 1 et 0 le rsultat est le mme qu'avec la fonction trans)

  (defun TransNested (pt rlst from to / mat dep geom)
    (setq mat '((1 0 0) (0 1 0) (0 0 1)))
    (and (= 1 from) (setq pt (trans pt 1 0)))
    (and (= 2 to) (setq rlst (reverse rlst)))
    (and (or (= 2 from) (= 2 to))
      (while rlst
	(setq geom (if (= 2 to)
		     (RevRefGeom (car rlst))
		     (RefGeom (car rlst))
		   )
	      rlst (cdr rlst)
	      mat (mxm (car geom) mat)
	      pt (mapcar '+ (mxv (car geom) pt) (cadr geom))
	)
      )
    )
    (if (= 1 to)
      (trans pt 0 1)
      pt
    )
  )

  ;-----------------------------------------------------------------
  ; Convertion du point par rapport au scu
  ;-----------------------------------------------------------------

  (defun conv(pt per fen)
    (if per
      (if (and (zerop (getvar "tilemode" ))(eq (vla-get-name lay) "*Model_Space"))
        (wcs2pcs (transnested pt (list (vlax-vla-object->ename per)) 2 3) fen)
	(transnested pt (list (vlax-vla-object->ename per)) 2 3)
      )
      (if (and (zerop (getvar "tilemode" ))(eq (vla-get-name lay) "*Model_Space"))
	(wcs2pcs pt fen)
	pt
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Dtermine si l'objet est visible ou pas
  ;-----------------------------------------------------------------

  (defun gele(ent cal per / bg cac ce fen hd ok)
    (setq cac (vla-item (vla-get-layers doc) cal))
    (and (eq (vla-get-layeron cac) :vlax-true)
	 (eq (vla-get-freeze cac) :vlax-false)
      (progn
	(if (zerop (getvar "tilemode"))
	  (cond
	    ((and (not (eq (vla-get-name lay) "*Model_Space"))
		  (eq (vla-get-name lay) (getvar "ctab")))
	      (setq ok (list (conv (vlax-get ent 'insertionpoint) per nil) nil))
	    )
	    (lst_fen
	      (foreach fen lst_fen
		(and (not ok)
		     (setq ce (conv (vlax-get ent 'insertionpoint) per (car fen))
			   bg (cadr fen)
			   hd (caddr fen)
		     )
		     (> (car  ce) (car  bg))
		     (> (cadr ce) (cadr bg))
		     (< (car  ce) (car  hd))
		     (< (cadr ce) (cadr hd))
		     (not (member cal (nth 3 fen)))
		     (setq ok (list ce (car fen)))
		)
	      )
	    )
	  )
	  (setq ok (list (conv (vlax-get ent 'insertionpoint) per nil) nil))
	)
      )
    )
    ok
  )

  ;-----------------------------------------------------------------
  ; Zoom sur l'objet
  ;-----------------------------------------------------------------

  (defun affiche(val per / bg di fact hauteur hd )
    (defun hauteur(val / ht1 ht2 ht3 loc pos str)
      (setq ht1 (vla-get-height val) ht3 0 pos 0 str (vla-get-textstring val))
      (while (setq loc (vl-string-search "\\H" str pos))
	(if (> (setq ht2 (atof (substr str (+ loc 3) (- (- (vl-string-search "x" str loc) 2) loc)))) ht3)
	  (setq ht3 ht2)
	)
	(setq pos (+ (vl-string-search "x" str loc) 2))
      )
      (if (/= ht3 0)
	(* ht1 ht3)
	ht1
      )
    )

    (and val
      (progn
	(initget "Zoom Suivant")
	(and (or (not (setq rep (getkword (strcat "\nTexte n " (itoa tot) " trouv " (unformat (vla-get-textstring val) "*") " (Zoom/Suivant) <Zoom> : "))))
	    	 (eq rep "Zoom")
	     )
	  (progn
	    (and (zerop (getvar "tilemode")) (eq (vla-get-mspace doc) :vlax-true)
	      (setq spa doc)
	      (vla-put-mspace doc :vlax-false)
	    )
	    (if per
	      (progn
		(vla-getboundingbox val 'bg 'hd)
		(setq bg (vlax-safearray->list bg)
		      hd (vlax-safearray->list hd)
		      fact (/ (distance (transnested bg (list (vlax-vla-object->ename per)) 0 1)
					(transnested hd (list (vlax-vla-object->ename per)) 0 1)
			      )
			      (distance bg hd)
			   )
		)
	      )
	      (setq fact 1.0)
	    )
	    (cond
	      ((not (zerop (getvar "tilemode")))
		(setq di (/ (hauteur val) 0.075))
	      )
	      ((and (zerop (getvar "tilemode")) (eq (vla-get-name lay) "*Model_Space"))
		(setq di (* (/ (hauteur val) 0.075) (vla-get-customscale (cadr cen))))
	      )
	      (T
		(setq di (/ (hauteur val) 0.075))
	      )
	    )
	    (and old_patrick:recherche
		 (vlax-read-enabled-p old_patrick:recherche)
		 (redraw (vlax-vla-object->ename old_patrick:recherche) 4)
	    )
	    (redraw (vlax-vla-object->ename val) 3)
	    (setq old_patrick:recherche val)
	    (vla-zoomcenter (vlax-get-acad-object) (vlax-3d-point (car cen)) (/ (* di 5) fact))
	  )
	)
	(setq tot (1+ tot))
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Dtermine le calque de l'objet
  ;-----------------------------------------------------------------

  (defun calque(ent cal)
    (cond
      ((eq (vla-get-layer ent) "0")
	cal
      )
      ((vl-string-search "|" cal)
	(strcat (substr cal 1 (1+ (vl-string-search "|" cal))) (vla-get-layer ent))
      )
      (T
	(vla-get-layer ent)
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Recherche Textes
  ;-----------------------------------------------------------------

  (defun rech_txt(ent / lst res)
    (if wcm
      (setq res (wcmatch (strcase (unformat (vla-get-textstring ent) "*")) (strcase str)))
      (setq res (eq (strcase (unformat (vla-get-textstring ent) "*")) (strcase str)))
    )
    (and res
      (setq lst ent)
    )
    lst
  )

  ;-----------------------------------------------------------------
  ; Recherche Attributs
  ;-----------------------------------------------------------------

  (defun rech_att(ent cal / att lst res)
    (foreach att (vlax-invoke ent 'getattributes)
      (if wcm
	(setq res (wcmatch (strcase (vla-get-textstring att)) (strcase str)))
	(setq res (eq (strcase (vla-get-textstring att)) (strcase str)))
      )
      (and res
	(setq lst (cons (list att (calque att cal)) lst) res nil)
      )
    )
    (foreach att (vlax-invoke ent 'getconstantattributes)
      (if wcm
	(setq res (wcmatch (strcase (vla-get-textstring att)) (strcase str)))
	(setq res (eq (strcase (vla-get-textstring att)) (strcase str)))
      )
      (and res
	(setq lst (cons (list att (calque att cal)) lst) res nil)
      )
    )
    (reverse lst)
  )

  ;-----------------------------------------------------------------
  ; Recherche Blocs
  ;-----------------------------------------------------------------

  (defun rech_bl(ent obj per cal / atr bl ech lst nom sub)
    (setq bl (list ent))
    (while bl
      (foreach atr (rech_att (car bl) cal)
	(and (setq cen (gele (car atr) (cadr atr) per))
	  (affiche (car atr) per)
	)
      )
      (if (vlax-property-available-p (car bl) 'effectivename)
	(setq nom (vla-get-effectivename (car bl)))
	(setq nom (vla-get-name (car bl)))
      )
      (vlax-for sub (vla-item (vla-get-blocks obj) nom)
	(if (eq (vla-get-objectname sub) "AcDbBlockReference")
	  (setq lst (cons sub lst))
	  (and (member (vla-get-objectname sub) '("AcDbText" "AcDbMText"))
	       (rech_txt sub)
	       (setq cen (gele sub (calque sub cal) (car bl)))
	       (affiche sub per)
	  )
	)
      )
      (setq bl (append (cdr bl) (reverse lst))
	    lst nil
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Recherche Tableaux
  ;-----------------------------------------------------------------

  (defun rech_tab(ent obj per cal / bl lst sub)
    (setq bl (list ent))
    (while bl
      (vlax-for sub (vla-item (vla-get-blocks obj) (cdr (assoc 2 (entget (vlax-vla-object->ename (car bl))))))
	(cond
	  ((eq (vla-get-objectname sub) "AcDbBlockReference")
	    (and (or (eq (logand flag 16) 16)
		     (and (eq (logand flag 4) 4)
			  (eq (logand flag 128) 128)
		     )
		 )
		 (rech_bl sub obj (car bl) cal)
	    )
	  )
	  ((member (vla-get-objectname sub) '("AcDbText" "AcDbMText"))
	    (and (or (eq (logand flag 8) 8)
		     (and (eq (logand flag 4) 4)
			  (eq (logand flag 64) 64)
		     )
		 )
		 (rech_txt sub)
		 (setq cen (gele sub (calque sub cal) (car bl)))
		 (affiche sub per)
	    )
	  )
	  ((eq (vla-get-objectname sub) "AcDbTable")
	    (and (or (eq (logand flag 32) 32)
		     (and (eq (logand flag 4) 4)
			  (eq (logand flag 256) 256)
		     )
		 )
		 (setq lst (cons (vlax-vla-object->ename sub) lst))
	    )
	  )
	)
      )
      (setq bl (append (cdr bl) (reverse lst))
	    lst nil
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Recherche Xrefs
  ;-----------------------------------------------------------------

  (defun rech_xref(ent / bl cal ech lst obj val xre)
    (setq bl (list ent))
    (while bl
      (and (setq val (findfile (vla-get-path (car bl))))
	   (not (zerop (vla-get-count (vla-item (vla-get-blocks doc) (vla-get-name (car bl))))))
	(progn
	  (if (vl-catch-all-error-p (setq xreo (vl-catch-all-apply 'vla-item (list (vla-get-documents (vlax-get-acad-object)) (strcat (vl-filename-base val)(vl-filename-extension val))))))
	    (setq dbx (ouvrir_dessin_dbx val) xreo nil)
	    (setq dbx xreo)
	  )
	  (vlax-for xre (setq obj (vla-item (vla-get-blocks dbx) "*Model_Space"))
	    (if (eq (vla-get-layer xre) "0")
	      (setq cal "0")
	      (setq cal (strcat (vla-get-name ent) "|" (vla-get-layer xre)))
	    )
	    (cond
	      ((eq (vla-get-objectname xre) "AcDbBlockReference")
		(if (vlax-property-available-p xre 'path)
		  (setq lst (cons xre lst))
		  (and	(eq (logand flag 128) 128)
			(rech_bl xre dbx (car bl) cal)
		  )
		)
	      )
	      ((member (vla-get-objectname xre) '("AcDbText" "AcDbMText"))
		(and (eq (logand flag 64) 64)
		     (rech_txt xre)
		     (setq cen (gele xre cal (car bl)))
		     (affiche xre (car bl))
		)
	      )
	      ((eq (vla-get-objectname xre) "AcDbTable")
		(and (eq (logand flag 256) 256)
		     (rech_tab xre dbx (car bl) cal)
		)
	      )
	    )
	  )
	  (or xreo (vlax-release-object dbx))
	  (setq dbx nil xreo nil val nil lst nil)
	)
      )
      (setq bl (append (cdr bl) (reverse lst))
	    lst nil
	    val nil
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Recherche globale
  ;-----------------------------------------------------------------

  (defun reche(ent)
    (cond
      ((eq (vla-get-objectname ent) "AcDbBlockReference")
	(if (vlax-property-available-p ent 'path)
	  (and (eq (logand flag 4) 4)
	       (rech_xref ent)
	  )
	  (and (eq (logand flag 16) 16)
	       (rech_bl ent doc nil (vla-get-layer ent))
	  )
	)
      )
      ((member (vla-get-objectname ent) '("AcDbText" "AcDbMText"))
	(and (eq (logand flag 8) 8)
	     (rech_txt ent)
	     (setq cen (gele ent (vla-get-layer ent) nil))
	     (affiche ent nil)
	)
      )
      ((eq (vla-get-objectname ent) "AcDbTable")
	(and (eq (logand flag 32) 32)
	     (rech_tab ent doc nil (vla-get-layer ent))
	)
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Routine principale
  ;-----------------------------------------------------------------

  (setq old_error *error*
	*error* *errpat*
	doc (vla-get-activedocument (vlax-get-acad-object))
	tot 1
  )
  (if (setq flag (getenv "Patrick_35_Rech"))
    (setq flag (atoi flag))
    (setq flag 511)
  )
  (vla-startundomark doc)
  (and (zerop (getvar "tilemode"))
    (progn
      (ssget "x" (list (cons 0 "VIEWPORT") (cons 410 (getvar "ctab")) (cons -4 "<NOT") (cons 69 1) (cons -4 "NOT>")))
      (vlax-for fen (setq sel (vla-get-activeselectionset doc))
	(vla-getboundingbox fen 'bg 'hd)
	(vla-getxdata fen "" 'cod 'cac)
	(setq lst (mapcar '(lambda (a b)(cons a (vlax-variant-value b)))(vlax-safearray->list cod)(vlax-safearray->list cac))
	      lst_fen (cons (list fen
				  (vlax-safearray->list bg)
				  (vlax-safearray->list hd)
				  (mapcar 'cdr (vl-remove-if-not '(lambda(a) (eq (car a) 1003)) lst))
			    )
			    lst_fen
		      )
	)
      )
      (vla-delete sel)
      (setq lst_fen (vl-sort lst_fen '(lambda(a b)(> (distance (cadr a)(caddr a))(distance (cadr b)(caddr b))))))
    )
  )
  (and	(or (eq (logand flag 1) 1)
	    (eq (logand flag 2) 2)
	)
	(or (eq (logand flag 4) 4)
	    (eq (logand flag 8) 8)
	    (eq (logand flag 16) 16)
	    (eq (logand flag 32) 32)
	)
	(or wcm (setq str (unformat str "*")))
    (progn
      (and (eq (logand flag 1) 1) (vlax-map-collection (setq lay (vla-item (vla-get-blocks doc) "*Model_Space")) 'reche))
      (setq spa nil)
      (and (eq (logand flag 2) 2) (zerop (getvar "tilemode"))
	(progn
	  (ssget "x" (list (cons 410 (getvar "ctab"))))
	  (setq lay (vla-item (vla-get-layouts doc) (getvar "ctab")))
	  (vlax-map-collection (setq sel (vla-get-activeselectionset doc)) 'reche)
	  (vla-delete sel)
	)
      )
    )
  )
  (vla-endundomark doc)
  (setq *error* old_error)
  (princ)
)

;;;=================================================================
;;;
;;; RECH.LSP V1.10
;;;
;;; Rechercher en tapant un texte
;;;
;;; Copyright (C) Patrick_35
;;;
;;;=================================================================

(defun c:rech(/ str)
  (vl-load-com)
  (and	(setq str (getstring T "\nTexte  rechercher : "))
	(/= str "")
	(patrick:rechercher str T)
  )
  (princ)
)

;;;=================================================================
;;;
;;; RECHT.LSP V1.10
;;;
;;; Rechercher en rcuperant la valeur texte d'un objet
;;;
;;; Copyright (C) Patrick_35
;;;
;;;=================================================================

(defun c:recht(/ str)
  (vl-load-com)
  (and	(princ "\nTexte  rechercher : ")
	(setq str (nentsel))
	(setq str (entget (car str)))
	(/= (cdr (assoc 1 str)) "")
	(member (cdr (assoc 0 str)) '("MTEXT" "TEXT" "ATTRIB"))
	(patrick:rechercher (cdr (assoc 1 str)) nil)
  )
  (princ)
)

;;;=================================================================
;;;
;;; RECHO.LSP V1.00
;;;
;;; Options de recherche d'un texte dans le dessin, dans les xrefs
;;;
;;; Copyright (C) Patrick_35
;;;
;;;=================================================================

(defun c:recho(/ dia lst tag msgbox flag xrefs)

  ;-----------------------------------------------------------------
  ; Afficher un message via une boite de dialogue
  ;-----------------------------------------------------------------

  (defun MsgBox (Titre Bouttons Message / Reponse WshShell)
    (acad-push-dbmod)
    (setq WshShell (vlax-create-object "WScript.Shell"))
    (setq Reponse (vlax-invoke WshShell 'Popup Message 0 Titre (itoa Bouttons)))
    (vlax-release-object WshShell)
    (acad-pop-dbmod)
    Reponse
  )

  ;-----------------------------------------------------------------
  ; Ajouter/Enlever un bit
  ;-----------------------------------------------------------------

  (defun flag(nb)
    (if (eq (logand tag nb) nb)
      (setq tag (- tag nb))
      (setq tag (+ tag nb))
    )
  )

  ;-----------------------------------------------------------------
  ; Griser ou pas les options textes
  ;-----------------------------------------------------------------

  (defun espace(/ xre)
    (if (or (eq (logand tag 1) 1)(eq (logand tag 2) 2))
      (progn
	(foreach xre '("xre" "txt" "att" "tab")
	  (mode_tile xre 0)
	)
	(xrefs)
      )
      (foreach xre '("xre" "txt" "att" "tab" "attxre" "tabxre" "txtxre")
	(mode_tile xre 1)
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Griser ou pas les options xrefs
  ;-----------------------------------------------------------------

  (defun xrefs(/ xre)
    (if (eq (logand tag 4) 4)
      (foreach xre '("attxre" "tabxre" "txtxre")
	(mode_tile xre 0)
      )
      (foreach xre '("attxre" "tabxre" "txtxre")
	(mode_tile xre 1)
      )
    )
  )

  ;-----------------------------------------------------------------
  ; Routine principale
  ;-----------------------------------------------------------------

  (vl-load-com)
  (if (setq dia (findfile "rech.dcl"))
    (progn
      (if (setq tag (getenv "Patrick_35_Rech"))
	(setq tag (atoi tag))
        (setq tag 511)
      )
      (setq dia (load_dialog dia))
      (new_dialog "rech" dia "")
      (foreach lst '((1 "eso") (2 "esp") (4 "xre") (8 "txt") (16 "att") (32 "tab") (64 "txtxre") (128 "attxre") (256 "tabxre"))
	(and (eq (logand tag (car lst)) (car lst))
	  (set_tile (cadr lst) "1")
	)
      )
      (espace)
      (set_tile "titre" "Options de Recherche V1.10")
      (action_tile "eso"    "(flag 1)(espace)")
      (action_tile "esp"    "(flag 2)(espace)")
      (action_tile "xre"    "(flag 4)(xrefs)")
      (action_tile "txt"    "(flag 8)")
      (action_tile "att"    "(flag 16)")
      (action_tile "tab"    "(flag 32)")
      (action_tile "txtxre" "(flag 64)")
      (action_tile "attxre" "(flag 128)")
      (action_tile "tabxre" "(flag 256)")
      (action_tile "accept" "(done_dialog 1)")
      (action_tile "cancel" "(done_dialog 0)")
      (and (eq (start_dialog) 1)
	(setenv "Patrick_35_Rech" (itoa tag))
      )
      (unload_dialog dia)
    )
    (msgbox "Options de Recherche V1.10" 16 "Le fichier \"RECH.DCL\" est introuvable")
  )
  (princ)
)

(setq nom_lisp "RECH, RECHT, RECHO")
(if (/= app nil)
  (if (= (strcase (substr app (1+ (- (strlen app) (strlen nom_lisp))) (strlen nom_lisp))) nom_lisp)
    (princ (strcat "..." nom_lisp " charg."))
    (princ (strcat "\n" nom_lisp " Charg......Faites " nom_lisp " pour excuter la fonction dsire.")))
  (princ (strcat "\n" nom_lisp " Charg......Faites " nom_lisp " pour excuter la fonction dsire.")))
(setq nom_lisp nil)
(princ)